@@ -1,8 +1,12 @@
-banner = ['TeXExec', 'version 6.2.0', '1997-2006', 'PRAGMA ADE/POD']
+#!/usr/bin/env ruby
+#encoding: ASCII-8BIT
+
+banner = ['TeXExec', 'version 6.2.1', '1997-2009', 'PRAGMA ADE/POD']
 
 $: << File.expand_path(File.dirname($0)) ; $: << File.join($:.last,'lib') ; $:.uniq!
 
-require 'ftools'     # needed ?
+require 'fileutils'
+# require 'ftools'     # needed ?
 
 require 'base/switch'
 require 'base/logger'
@@ -111,7 +115,7 @@ class Commands
         if job = TEX.new(logger) then
             prepare(job)
             job.cleanuptemprunfiles
-            files = @commandline.arguments.sort
+            files =  if @commandline.option('sort') then @commandline.arguments.sort else @commandline.arguments end
             if files.length > 0 then
                 if f = File.open(job.tempfilename('tex'),'w') then
                     backspace = @commandline.checkedoption('backspace', '1.5cm')
@@ -156,11 +160,12 @@ class Commands
             prepare(job)
             job.cleanuptemprunfiles
             fast = @commandline.option('fast')
-            files = @commandline.arguments.sort
+            files =  if @commandline.option('sort') then @commandline.arguments.sort else @commandline.arguments end
             if fast or (files.length > 0) then
                 if f = File.open(job.tempfilename('tex'),'w') then
                     files.delete("texexec.pdf")
-                    Kpse.runscript('rlxtools', ['--identify','--collect'], files.join(' ')) unless fast
+                    # Kpse.runscript('rlxtools', ['--identify','--collect'], files.join(' ')) unless fast
+                    system("texmfstart rlxtools --identify --collect #{files.join(' ')}")
                     figures     = @commandline.checkedoption('method', 'a').downcase
                     paperoffset = @commandline.checkedoption('paperoffset', '0pt')
                     backspace   = @commandline.checkedoption('backspace', '1.5cm')
@@ -202,7 +207,7 @@ class Commands
         if job = TEX.new(logger) then
             prepare(job)
             job.cleanuptemprunfiles
-            files = @commandline.arguments.sort
+            files =  if @commandline.option('sort') then @commandline.arguments.sort else @commandline.arguments end
             msuffixes = ['tex','mkii','mkiv','mp','pl','pm','rb']
             if files.length > 0 then
                 files.each do |fname|
@@ -222,7 +227,8 @@ class Commands
                                 else
                                     markfile = nil
                                 end
-                                Kpse.runscript('ctxtools',ffname,'--document')
+                                # Kpse.runscript('ctxtools',['--document'],ffname)
+                                system("texmfstart ctxtools --document #{ffname}")
                                 if ted = File.silentopen(File.suffixed(ffname,'ted')) then
                                     firstline = ted.gets
                                     if firstline =~ /interface=/o then
@@ -277,6 +283,7 @@ class Commands
                     info = `pdfinfo #{filename}`
                     if info =~ /Pages:\s*(\d+)/ then
                         nofpages = $1.to_i
+                        result = @commandline.checkedoption('result','texexec')
                         nofpages.times do |i|
                             if f = File.open(tempfile,"w") then
                                 n = i + 1
@@ -285,8 +292,10 @@ class Commands
                                 f << "\\externalfigure[#{filename}][object=no,page=#{n}]\n"
                                 f << "\\stopTEXpage\\stoptext\n"
                                 f.close
+                                job.setvariable('result',"#{result}-#{n}")
                                 job.setvariable('interface','english') # redundant
                                 job.setvariable('simplerun',true)
+                                job.setvariable('purge',true)
                                 job.setvariable('files',[tempfile])
                                 job.processtex
                             end
@@ -302,7 +311,7 @@ class Commands
         if job = TEX.new(logger) then
             prepare(job)
             job.cleanuptemprunfiles
-            files = @commandline.arguments.sort
+            files =  if @commandline.option('sort') then @commandline.arguments.sort else @commandline.arguments end
             if files.length > 0 then
                 if f = File.open(job.tempfilename('tex'),'w') then
                     emptypages  = @commandline.checkedoption('addempty', '')
@@ -355,7 +364,7 @@ class Commands
         if job = TEX.new(logger) then
             prepare(job)
             job.cleanuptemprunfiles
-            files = @commandline.arguments.sort
+            files =  if @commandline.option('sort') then @commandline.arguments.sort else @commandline.arguments end
             if files.length > 0 then
                 if f = File.open(job.tempfilename('tex'),'w') then
                     selection   = @commandline.checkedoption('selection', '')
@@ -425,7 +434,7 @@ class Commands
         if job = TEX.new(logger) then
             prepare(job)
             job.cleanuptemprunfiles
-            files = @commandline.arguments.sort
+            files =  if @commandline.option('sort') then @commandline.arguments.sort else @commandline.arguments end
             if files.length > 0 then
                 if f = File.open(job.tempfilename('tex'),'w') then
                     scale = @commandline.checkedoption('scale')
@@ -492,7 +501,7 @@ class Commands
         if job = TEX.new(logger) then
             prepare(job)
             job.cleanuptemprunfiles
-            files = @commandline.arguments.sort
+            files =  if @commandline.option('sort') then @commandline.arguments.sort else @commandline.arguments end
             if files.length > 0 then
                 if f = File.open(job.tempfilename('tex'),'w') then
                     paperoffset = @commandline.checkedoption('paperoffset', '0cm')
@@ -575,14 +584,16 @@ job.setvariable('given.backend',job.getvariable('backend'))
 
         if (str = @commandline.option('engine')) && ! str.standard? && ! str.empty? then
             job.setvariable('texengine',str)
+        elsif @commandline.oneof('luatex') then
+            job.setvariable('texengine','luatex')
         elsif @commandline.oneof('pdfetex','pdftex','pdf') then
             job.setvariable('texengine','pdftex')
         elsif @commandline.oneof('xetex','xtx') then
             job.setvariable('texengine','xetex')
         elsif @commandline.oneof('aleph') then
             job.setvariable('texengine','aleph')
-        elsif @commandline.oneof('luatex') then
-            job.setvariable('texengine','luatex')
+        elsif @commandline.oneof('petex') then
+            job.setvariable('texengine','petex')
         else
             job.setvariable('texengine','standard')
         end
@@ -597,6 +608,8 @@ job.setvariable('given.backend',job.getvariable('backend'))
             job.setvariable('backend','xetex')
         elsif @commandline.oneof('aleph') then
             job.setvariable('backend','dvipdfmx')
+        elsif @commandline.oneof('petex') then
+            job.setvariable('backend','dvipdfmx')
         elsif @commandline.oneof('dvips','ps') then
             job.setvariable('backend','dvips')
         elsif @commandline.oneof('xdv') then
@@ -607,6 +620,7 @@ job.setvariable('given.backend',job.getvariable('backend'))
                 when 'pdftex'   then job.setvariable('backend','pdftex')
                 when 'luatex'   then job.setvariable('backend','pdftex')
                 when 'xetex'    then job.setvariable('backend','xetex')
+                when 'petex'    then job.setvariable('backend','dvipdfmx')
                 when 'aleph'    then job.setvariable('backend','dvipdfmx')
             else
                 job.setvariable('backend','standard')
@@ -760,8 +774,11 @@ commandline.registerflag('xdv')
 
 commandline.registerflag('aleph')
 
+commandline.registerflag('petex')
+
 commandline.registerflag('all')
 commandline.registerflag('fast')
+commandline.registerflag('sort')
 
 # generic
 
