@@ -1,11 +1,13 @@
 if not modules then modules = { } end modules ['char-cmp'] = {
     version   = 1.001,
-    comment   = "companion to char-ini.tex",
+    comment   = "companion to char-ini.mkiv",
     author    = "Hans Hagen, PRAGMA-ADE, Hasselt NL",
     copyright = "PRAGMA ADE / ConTeXt Development Team",
     license   = "see context related readme files"
 }
 
+local utf = unicode.utf8
+
 characters            = characters            or { }
 characters.uncomposed = characters.uncomposed or { }
 
